@@ -241,7 +241,7 @@ class _BaseEventedItemModel(QAbstractItemModel, Generic[ItemType]):
         par, idx = self._split_nested_index(event.index)
         self.beginInsertRows(par, idx, idx)
 
-    def _on_end_insert(self, e):
+    def _on_end_insert(self):
         """Must be called after insert operation to update model."""
         self.endInsertRows()
 
@@ -254,7 +254,7 @@ class _BaseEventedItemModel(QAbstractItemModel, Generic[ItemType]):
         par, idx = self._split_nested_index(event.index)
         self.beginRemoveRows(par, idx, idx)
 
-    def _on_end_remove(self, e):
+    def _on_end_remove(self):
         """Must be called after remove operation to update model."""
         self.endRemoveRows()
 
@@ -269,7 +269,7 @@ class _BaseEventedItemModel(QAbstractItemModel, Generic[ItemType]):
 
         self.beginMoveRows(src_par, src_idx, src_idx, dest_par, dest_idx)
 
-    def _on_end_move(self, e):
+    def _on_end_move(self):
         """Must be called after move operation to update model."""
         self.endMoveRows()
 
