@@ -7,6 +7,7 @@ from napari_plugin_engine.dist import standard_metadata
 from qtpy.QtCore import (
     QEvent,
     QObject,
+    QPoint,
     QProcess,
     QProcessEnvironment,
     QSize,
@@ -45,7 +46,9 @@ from ...plugins.pypi import (
 from ...utils._appdirs import user_plugin_dir, user_site_packages
 from ...utils.misc import parse_version, running_as_bundled_app
 from ...utils.translations import trans
+from ..qt_resources import QColoredSVGIcon
 from ..qthreading import create_worker
+from ..widgets.qt_message_popup import WarnPopup
 
 InstallerTypes = Literal['pip', 'conda', 'mamba']
 
@@ -310,6 +313,7 @@ class PluginListItem(QFrame):
         parent: QWidget = None,
         enabled: bool = True,
         installed: bool = False,
+        npe_version=1,
     ):
         super().__init__(parent)
         self.setup_ui(enabled)
@@ -319,6 +323,11 @@ class PluginListItem(QFrame):
         self.package_author.setText(author)
         self.cancel_btn.setVisible(False)
 
+        self.help_button.setText(trans._("Website"))
+        self.help_button.setObjectName("help_button")
+        if npe_version != 1:
+            self._handle_npe2_plugin()
+
         if installed:
             self.enabled_checkbox.show()
             self.action_button.setText(trans._("uninstall"))
@@ -328,6 +337,17 @@ class PluginListItem(QFrame):
             self.action_button.setText(trans._("install"))
             self.action_button.setObjectName("install_button")
 
+    def _handle_npe2_plugin(self):
+        npe2_icon = QLabel(self)
+        icon = QColoredSVGIcon.from_resources('logo_silhouette')
+        npe2_icon.setPixmap(icon.colored(color='#33F0FF').pixmap(20, 20))
+        self.row1.insertWidget(2, QLabel('npe2'))
+        self.row1.insertWidget(2, npe2_icon)
+        self.enabled_checkbox.setEnabled(False)
+        self.enabled_checkbox.setToolTip(
+            'This is a npe2 plugin and cannot be enabled/disabled at this time.'
+        )
+
     def _get_dialog(self) -> QDialog:
         p = self.parent()
         while not isinstance(p, QDialog) and p.parent():
@@ -347,7 +367,6 @@ class PluginListItem(QFrame):
         self.v_lay.setContentsMargins(-1, 6, -1, 6)
         self.v_lay.setSpacing(0)
         self.row1 = QHBoxLayout()
-        self.row1.setSpacing(6)
         self.enabled_checkbox = QCheckBox(self)
         self.enabled_checkbox.setChecked(enabled)
         self.enabled_checkbox.stateChanged.connect(self._on_enabled_checkbox)
@@ -397,6 +416,7 @@ class PluginListItem(QFrame):
         self.update_btn.setObjectName("install_button")
         self.row1.addWidget(self.update_btn)
         self.update_btn.setVisible(False)
+        self.help_button = QPushButton(self)
         self.action_button = QPushButton(self)
         sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
         sizePolicy.setHorizontalStretch(0)
@@ -404,7 +424,9 @@ class PluginListItem(QFrame):
         sizePolicy.setHeightForWidth(
             self.action_button.sizePolicy().hasHeightForWidth()
         )
+        self.help_button.setSizePolicy(sizePolicy)
         self.action_button.setSizePolicy(sizePolicy)
+        self.row1.addWidget(self.help_button)
         self.row1.addWidget(self.action_button)
         self.v_lay.addLayout(self.row1)
         self.row2 = QHBoxLayout()
@@ -461,6 +483,7 @@ class QPluginList(QListWidget):
         installed=False,
         plugin_name=None,
         enabled=True,
+        npe_version=1,
     ):
         # don't add duplicates
         if (
@@ -474,19 +497,29 @@ class QPluginList(QListWidget):
         item = QListWidgetItem(searchable_text, parent=self)
         item.version = project_info.version
         super().addItem(item)
-
         widg = PluginListItem(
             *project_info,
             parent=self,
             plugin_name=plugin_name,
             enabled=enabled,
             installed=installed,
+            npe_version=npe_version,
         )
         item.widget = widg
+        item.npe_version = npe_version
         action_name = 'uninstall' if installed else 'install'
         item.setSizeHint(widg.sizeHint())
         self.setItemWidget(item, widg)
 
+        if project_info.url:
+            import webbrowser
+
+            widg.help_button.clicked.connect(
+                lambda: webbrowser.open(project_info.url)
+            )
+        else:
+            widg.help_button.setVisible(False)
+
         widg.action_button.clicked.connect(
             lambda: self.handle_action(item, project_info.name, action_name)
         )
@@ -506,6 +539,20 @@ class QPluginList(QListWidget):
         item.setText("0-" + item.text())
         method = getattr(self.installer, action_name)
         self._remove_list.append((pkg_name, item))
+        self._warn_dialog = None
+        if item.npe_version != 1:
+            # show warning pop up dialog
+            message = 'When installing/uninstalling npe2 plugins, you must restart napari for UI changes to take effect.'
+            self._warn_dialog = WarnPopup(
+                text=message,
+            )
+
+            delta_x = 75
+            global_point = widget.action_button.mapToGlobal(
+                widget.action_button.rect().topLeft()
+            )
+            global_point = QPoint(global_point.x() - delta_x, global_point.y())
+            self._warn_dialog.move(global_point)
 
         if action_name == "install":
             if update:
@@ -515,11 +562,15 @@ class QPluginList(QListWidget):
                 widget.set_busy(trans._("installing..."), update)
 
             method([pkg_name])
+            if self._warn_dialog:
+                self._warn_dialog.exec_()
             self.scrollToTop()
         elif action_name == "uninstall":
             widget.set_busy(trans._("uninstalling..."), update)
             widget.update_btn.setDisabled(True)
             method([pkg_name])
+            if self._warn_dialog:
+                self._warn_dialog.exec_()
             self.scrollToTop()
         elif action_name == "cancel":
             widget.set_busy(trans._("cancelling..."), update)
@@ -554,6 +605,7 @@ class QPluginList(QListWidget):
 class QtPluginDialog(QDialog):
     def __init__(self, parent=None):
         super().__init__(parent)
+        self.already_installed = set()
         self.installer = Installer()
         self.setup_ui()
         self.installer.set_output_widget(self.stdout_text)
@@ -587,23 +639,19 @@ class QtPluginDialog(QDialog):
         self.available_list.clear()
 
         # fetch installed
-        from ...plugins import plugin_manager
+        from ...plugins import _npe2, plugin_manager
 
         plugin_manager.discover()  # since they might not be loaded yet
 
-        already_installed = set()
-
-        for plugin_name, mod_name, distname in plugin_manager.iter_available():
-            # not showing these in the plugin dialog
-            if plugin_name in ('napari_plugin_engine',):
-                continue
-
-            if distname in already_installed:
-                continue
+        self.already_installed = set()
 
+        def _add_to_installed(distname, enabled, npe_version=1):
             if distname:
-                already_installed.add(distname)
                 meta = standard_metadata(distname)
+                if len(meta) == 0:
+                    # will not add builtins.
+                    return
+                self.already_installed.add(distname)
             else:
                 meta = {}
 
@@ -617,24 +665,37 @@ class QtPluginDialog(QDialog):
                     meta.get('license', ''),
                 ),
                 installed=True,
-                enabled=not plugin_manager.is_blocked(plugin_name),
+                enabled=enabled,
+                npe_version=npe_version,
             )
+
+        for manifest in _npe2.iter_manifests():
+            distname = normalized_name(manifest.name or '')
+            if distname in self.already_installed or distname == 'napari':
+                continue
+            _add_to_installed(distname, True, npe_version=2)
+
+        for plugin_name, mod_name, distname in plugin_manager.iter_available():
+            # not showing these in the plugin dialog
+            if plugin_name in ('napari_plugin_engine',):
+                continue
+            if distname in self.already_installed:
+                continue
+            _add_to_installed(
+                distname, not plugin_manager.is_blocked(plugin_name)
+            )
+
         self.installed_label.setText(
             trans._(
-                "Installed Plugins ({amount})", amount=len(already_installed)
+                "Installed Plugins ({amount})",
+                amount=len(self.already_installed),
             )
         )
 
         # fetch available plugins
         self.worker = create_worker(iter_napari_plugin_info)
 
-        def _handle_yield(project_info):
-            if project_info.name in already_installed:
-                self.installed_list.tag_outdated(project_info)
-            else:
-                self.available_list.addItem(project_info)
-
-        self.worker.yielded.connect(_handle_yield)
+        self.worker.yielded.connect(self._handle_yield)
         self.worker.finished.connect(self.working_indicator.hide)
         self.worker.finished.connect(self._update_count_in_label)
         self.worker.start()
@@ -776,6 +837,24 @@ class QtPluginDialog(QDialog):
         if packages:
             self.installer.install(packages)
 
+    def _handle_yield(self, project_info):
+        if project_info.name in self.already_installed:
+            self.installed_list.tag_outdated(project_info)
+        else:
+            self.available_list.addItem(project_info)
+
+        self.filter()
+
+    def filter(self, text: str = None) -> None:
+        """Filter by text or set current text as filter."""
+        if text is None:
+            text = self.packages_filter.text()
+        else:
+            self.packages_filter.setText(text)
+
+        self.installed_list.filter(text)
+        self.available_list.filter(text)
+
 
 if __name__ == "__main__":
     from qtpy.QtWidgets import QApplication
