@@ -89,8 +89,11 @@ class QtBaseImageControls(QtLayerControls):
 
         # Create contrast_limits slider
         self.contrastLimitsSlider = _QDoubleRangeSlider(Qt.Horizontal, self)
-        self.contrastLimitsSlider.setSingleStep(0.01)
+        decimals = range_to_decimals(
+            self.layer.contrast_limits_range, self.layer.dtype
+        )
         self.contrastLimitsSlider.setRange(*self.layer.contrast_limits_range)
+        self.contrastLimitsSlider.setSingleStep(10 ** -decimals)
         self.contrastLimitsSlider.setValue(self.layer.contrast_limits)
         self.contrastLimitsSlider.setToolTip(
             trans._('Right click for detailed slider popup.')
@@ -135,14 +138,8 @@ class QtBaseImageControls(QtLayerControls):
         """
         self.layer.colormap = self.colormapComboBox.currentData()
 
-    def _on_contrast_limits_change(self, event=None):
-        """Receive layer model contrast limits change event and update slider.
-
-        Parameters
-        ----------
-        event : napari.utils.event.Event, optional
-            The napari event that triggered this method, by default None.
-        """
+    def _on_contrast_limits_change(self):
+        """Receive layer model contrast limits change event and update slider."""
         with qt_signals_blocked(self.contrastLimitsSlider):
             self.contrastLimitsSlider.setRange(
                 *self.layer.contrast_limits_range
@@ -154,14 +151,8 @@ class QtBaseImageControls(QtLayerControls):
             with qt_signals_blocked(self.clim_popup.slider):
                 self.clim_popup.slider.setValue(self.layer.contrast_limits)
 
-    def _on_colormap_change(self, event=None):
-        """Receive layer model colormap change event and update dropdown menu.
-
-        Parameters
-        ----------
-        event : napari.utils.event.Event, optional
-            The napari event that triggered this method, by default None.
-        """
+    def _on_colormap_change(self):
+        """Receive layer model colormap change event and update dropdown menu."""
         name = self.layer.colormap.name
         if name not in self.colormapComboBox._allitems:
             cm = AVAILABLE_COLORMAPS.get(name)
@@ -183,14 +174,8 @@ class QtBaseImageControls(QtLayerControls):
         )
         self.colorbarLabel.setPixmap(QPixmap.fromImage(image))
 
-    def _on_gamma_change(self, event=None):
-        """Receive the layer model gamma change event and update the slider.
-
-        Parameters
-        ----------
-        event : napari.utils.event.Event, optional
-            The napari event that triggered this method, by default None.
-        """
+    def _on_gamma_change(self):
+        """Receive the layer model gamma change event and update the slider."""
         with qt_signals_blocked(self.gammaSlider):
             self.gammaSlider.setValue(self.layer.gamma)
 
@@ -213,10 +198,10 @@ class AutoScaleButtons(QWidget):
         self.setLayout(QHBoxLayout())
         self.layout().setSpacing(2)
         self.layout().setContentsMargins(0, 0, 0, 0)
-        once_btn = QPushButton('once')
+        once_btn = QPushButton(trans._('once'))
         once_btn.setFocusPolicy(Qt.NoFocus)
 
-        auto_btn = QPushButton('continuous')
+        auto_btn = QPushButton(trans._('continuous'))
         auto_btn.setCheckable(True)
         auto_btn.setFocusPolicy(Qt.NoFocus)
         once_btn.clicked.connect(lambda: auto_btn.setChecked(False))
@@ -232,18 +217,7 @@ class QContrastLimitsPopup(QRangeSliderPopup):
     def __init__(self, layer: Image, parent=None):
         super().__init__(parent)
 
-        if np.issubdtype(layer.dtype, np.integer):
-            decimals = 0
-        else:
-            # scale precision with the log of the data range order of magnitude
-            # eg.   0 - 1   (0 order of mag)  -> 3 decimal places
-            #       0 - 10  (1 order of mag)  -> 2 decimals
-            #       0 - 100 (2 orders of mag) -> 1 decimal
-            #       ≥ 3 orders of mag -> no decimals
-            # no more than 6 decimals
-            d_range = np.subtract(*layer.contrast_limits_range[::-1])
-            decimals = min(6, max(int(3 - np.log10(d_range)), 0))
-
+        decimals = range_to_decimals(layer.contrast_limits_range, layer.dtype)
         self.slider.setRange(*layer.contrast_limits_range)
         self.slider.setDecimals(decimals)
         self.slider.setSingleStep(10 ** -decimals)
@@ -277,3 +251,37 @@ class QContrastLimitsPopup(QRangeSliderPopup):
             range_btn.setFixedWidth(65)
             range_btn.clicked.connect(layer.reset_contrast_limits_range)
             self._layout.addWidget(range_btn, alignment=Qt.AlignBottom)
+
+
+def range_to_decimals(range_, dtype):
+    """Convert a range to decimals of precision.
+
+    Parameters
+    ----------
+    range : tuple
+        Slider range, min and then max values.
+    dtype : np.dtype
+        Data type of the layer. Integers layers are given integer.
+        step sizes.
+
+    Returns
+    -------
+    int
+        Decimals of precision.
+    """
+
+    if hasattr(dtype, 'numpy_dtype'):
+        # retrieve the corresponding numpy.dtype from a tensorstore.dtype
+        dtype = dtype.numpy_dtype
+
+    if np.issubdtype(dtype, np.integer):
+        return 0
+    else:
+        # scale precision with the log of the data range order of magnitude
+        # eg.   0 - 1   (0 order of mag)  -> 3 decimal places
+        #       0 - 10  (1 order of mag)  -> 2 decimals
+        #       0 - 100 (2 orders of mag) -> 1 decimal
+        #       ≥ 3 orders of mag -> no decimals
+        # no more than 64 decimals
+        d_range = np.subtract(*range_[::-1])
+        return min(64, max(int(3 - np.log10(d_range)), 0))
