@@ -2,7 +2,8 @@ import numpy as np
 from qtpy.QtCore import Qt
 from qtpy.QtWidgets import QComboBox, QDoubleSpinBox, QLabel
 
-from ...layers.vectors._vectors_constants import ColorMode
+from ...layers.utils._color_manager_constants import ColorMode
+from ...utils.translations import trans
 from ..utils import qt_signals_blocked
 from ..widgets.qt_color_swatch import QColorSwatchEdit
 from .qt_layer_controls_base import QtLayerControls
@@ -44,8 +45,8 @@ class QtVectorsControls(QtLayerControls):
     def __init__(self, layer):
         super().__init__(layer)
 
-        self.layer.events.edge_width.connect(self._on_width_change)
-        self.layer.events.length.connect(self._on_len_change)
+        self.layer.events.edge_width.connect(self._on_edge_width_change)
+        self.layer.events.length.connect(self._on_length_change)
         self.layer.events.edge_color_mode.connect(
             self._on_edge_color_mode_change
         )
@@ -57,20 +58,23 @@ class QtVectorsControls(QtLayerControls):
         color_prop_box.activated[str].connect(self.change_edge_color_property)
         color_prop_box.addItems(color_properties)
         self.color_prop_box = color_prop_box
-        self.edge_prop_label = QLabel('edge property:')
+        self.edge_prop_label = QLabel(trans._('edge property:'))
 
         # vector direct color mode adjustment and widget
         self.edgeColorEdit = QColorSwatchEdit(
             initial_color=self.layer.edge_color,
-            tooltip='click to set current edge color',
+            tooltip=trans._(
+                'click to set current edge color',
+            ),
         )
         self.edgeColorEdit.color_changed.connect(self.change_edge_color_direct)
-        self.edge_color_label = QLabel('edge color:')
+        self.edge_color_label = QLabel(trans._('edge color:'))
         self._on_edge_color_change()
 
         # dropdown to select the edge color mode
         colorModeComboBox = QComboBox(self)
-        colorModeComboBox.addItems(ColorMode.keys())
+        color_modes = [e.value for e in ColorMode]
+        colorModeComboBox.addItems(color_modes)
         colorModeComboBox.activated[str].connect(self.change_edge_color_mode)
         self.color_mode_comboBox = colorModeComboBox
         self._on_edge_color_mode_change()
@@ -80,6 +84,7 @@ class QtVectorsControls(QtLayerControls):
         self.widthSpinBox.setKeyboardTracking(False)
         self.widthSpinBox.setSingleStep(0.1)
         self.widthSpinBox.setMinimum(0.1)
+        self.widthSpinBox.setMaximum(np.inf)
         self.widthSpinBox.setValue(self.layer.edge_width)
         self.widthSpinBox.valueChanged.connect(self.change_width)
 
@@ -89,19 +94,20 @@ class QtVectorsControls(QtLayerControls):
         self.lengthSpinBox.setSingleStep(0.1)
         self.lengthSpinBox.setValue(self.layer.length)
         self.lengthSpinBox.setMinimum(0.1)
+        self.lengthSpinBox.setMaximum(np.inf)
         self.lengthSpinBox.valueChanged.connect(self.change_length)
 
         # grid_layout created in QtLayerControls
         # addWidget(widget, row, column, [row_span, column_span])
-        self.grid_layout.addWidget(QLabel('opacity:'), 0, 0)
+        self.grid_layout.addWidget(QLabel(trans._('opacity:')), 0, 0)
         self.grid_layout.addWidget(self.opacitySlider, 0, 1, 1, 2)
-        self.grid_layout.addWidget(QLabel('width:'), 1, 0)
+        self.grid_layout.addWidget(QLabel(trans._('width:')), 1, 0)
         self.grid_layout.addWidget(self.widthSpinBox, 1, 1, 1, 2)
-        self.grid_layout.addWidget(QLabel('length:'), 2, 0)
+        self.grid_layout.addWidget(QLabel(trans._('length:')), 2, 0)
         self.grid_layout.addWidget(self.lengthSpinBox, 2, 1, 1, 2)
-        self.grid_layout.addWidget(QLabel('blending:'), 3, 0)
+        self.grid_layout.addWidget(QLabel(trans._('blending:')), 3, 0)
         self.grid_layout.addWidget(self.blendComboBox, 3, 1, 1, 2)
-        self.grid_layout.addWidget(QLabel('edge color mode:'), 4, 0)
+        self.grid_layout.addWidget(QLabel(trans._('edge color mode:')), 4, 0)
         self.grid_layout.addWidget(self.color_mode_comboBox, 4, 1, 1, 2)
         self.grid_layout.addWidget(self.edge_color_label, 5, 0)
         self.grid_layout.addWidget(self.edgeColorEdit, 5, 1, 1, 2)
@@ -186,7 +192,7 @@ class QtVectorsControls(QtLayerControls):
         self.setFocus()
 
     def _update_edge_color_gui(self, mode: str):
-        """ Update the GUI element associated with edge_color.
+        """Update the GUI element associated with edge_color.
         This is typically used when edge_color_mode changes
 
         Parameters
@@ -214,47 +220,29 @@ class QtVectorsControls(QtLayerControls):
         -------
         property_values : np.ndarray
             array of all of the union of the property names (keys)
-            in Vectors.properties and Vectors._property_choices
+            in Vectors.properties and Vectors.property_choices
 
         """
-        property_choices = [*self.layer._property_choices]
+        property_choices = [*self.layer.property_choices]
         properties = [*self.layer.properties]
         property_values = np.union1d(property_choices, properties)
 
         return property_values
 
-    def _on_len_change(self, event=None):
-        """Change length of vectors.
-
-        Parameters
-        ----------
-        event : napari.utils.event.Event, optional
-            The napari event that triggered this method, by default None.
-        """
+    def _on_length_change(self):
+        """Change length of vectors."""
         with self.layer.events.length.blocker():
             self.lengthSpinBox.setValue(self.layer.length)
 
-    def _on_width_change(self, event=None):
-        """"Receive layer model width change event and update width spinbox.
-
-        Parameters
-        ----------
-        event : napari.utils.event.Event, optional
-            The napari event that triggered this method, by default None.
-        """
+    def _on_edge_width_change(self):
+        """Receive layer model width change event and update width spinbox."""
         with self.layer.events.edge_width.blocker():
             self.widthSpinBox.setValue(self.layer.edge_width)
 
-    def _on_edge_color_mode_change(self, event=None):
-        """"Receive layer model edge color mode change event & update dropdown.
-
-        Parameters
-        ----------
-        event : napari.utils.event.Event, optional
-            The napari event that triggered this method, by default None.
-        """
+    def _on_edge_color_mode_change(self):
+        """Receive layer model edge color mode change event & update dropdown."""
         with qt_signals_blocked(self.color_mode_comboBox):
-            mode = self.layer.edge_color_mode
+            mode = self.layer._edge.color_mode
             index = self.color_mode_comboBox.findText(
                 mode, Qt.MatchFixedString
             )
@@ -262,22 +250,19 @@ class QtVectorsControls(QtLayerControls):
 
             self._update_edge_color_gui(mode)
 
-    def _on_edge_color_change(self, event=None):
-        """"Receive layer model edge color  change event & update dropdown.
-
-        Parameters
-        ----------
-        event : napari.utils.event.Event, optional
-            The napari event that triggered this method, by default None.
-        """
-        if self.layer._edge_color_mode == ColorMode.DIRECT:
+    def _on_edge_color_change(self):
+        """Receive layer model edge color  change event & update dropdown."""
+        if (
+            self.layer._edge.color_mode == ColorMode.DIRECT
+            and len(self.layer.data) > 0
+        ):
             with qt_signals_blocked(self.edgeColorEdit):
                 self.edgeColorEdit.setColor(self.layer.edge_color[0])
-        elif self.layer._edge_color_mode in (
+        elif self.layer._edge.color_mode in (
             ColorMode.CYCLE,
             ColorMode.COLORMAP,
         ):
             with qt_signals_blocked(self.color_prop_box):
-                prop = self.layer._edge_color_property
+                prop = self.layer._edge.color_properties.name
                 index = self.color_prop_box.findText(prop, Qt.MatchFixedString)
                 self.color_prop_box.setCurrentIndex(index)
