@@ -3,7 +3,7 @@ import ast
 import numpy as np
 import pytest
 
-from napari.components.layerlist import _CONTEXT_KEYS, LayerList
+from napari.components.layerlist import LayerList
 from napari.layers import Image, Labels
 from napari.layers._layer_actions import (
     _LAYER_ACTIONS,
@@ -12,6 +12,8 @@ from napari.layers._layer_actions import (
     _convert_dtype,
     _project,
 )
+from napari.utils.context._expressions import Expr
+from napari.utils.context._layerlist_context import LayerListContextKeys
 
 
 class assert_expression_variables(ast.NodeVisitor):
@@ -29,18 +31,21 @@ def test_layer_actions():
     """Test that all variables used in layer actions expressions are
     keys in CONTEXT_KEYS.
     """
-    names = set(_CONTEXT_KEYS.keys())
+    names = set(LayerListContextKeys.__members__)
     valid_keys = set.union(
         set(ContextAction.__annotations__), set(SubMenu.__annotations__)
     )
     for action_dict in _LAYER_ACTIONS:
         for action in action_dict.values():
             assert set(action).issubset(valid_keys)
-            expr = action.get('enable_when', None)
-            if expr:
+            expr = action.get('enable_when')
+            if not expr:
+                continue
+            assert isinstance(expr, (bool, Expr))
+            if isinstance(expr, Expr):
                 assert_expression_variables(expr, names)
             expr = action.get('show_when', None)
-            if expr:
+            if isinstance(expr, Expr):
                 assert_expression_variables(expr, names)
 
 
