@@ -1,6 +1,11 @@
 """ImageView class.
 """
-from ...types import ArrayLike, Callable
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from ...types import ArrayLike, Callable
 
 
 class ImageView:
