@@ -1,11 +1,11 @@
 import dask.array as da
 import numpy as np
 import pytest
-import tensorstore as ts
 import xarray as xr
 
 from napari._tests.utils import check_layer_world_data_extent
 from napari.layers import Image
+from napari.layers.image._image_constants import ImageRendering
 from napari.layers.utils.plane import ClippingPlaneList, SlicingPlane
 from napari.utils import Colormap
 from napari.utils.transforms.transform_utils import rotate_to_matrix
@@ -554,6 +554,7 @@ def test_message_3d():
     np.random.seed(0)
     data = np.random.random((10, 15, 15))
     layer = Image(data)
+    layer._ndisplay = 3
     msg = layer.get_status(
         (0, 0, 0), view_direction=[1, 0, 0], dims_displayed=[0, 1, 2]
     )
@@ -654,8 +655,8 @@ def test_grid_translate():
     data = np.random.random((10, 15))
     layer = Image(data)
     translate = np.array([15, 15])
-    layer.translate_grid = translate
-    np.testing.assert_allclose(layer.translate_grid, translate)
+    layer._translate_grid = translate
+    np.testing.assert_allclose(layer._translate_grid, translate)
 
 
 def test_world_data_extent():
@@ -787,8 +788,50 @@ def test_instantiate_with_experimental_clipping_planes_dict():
 
 def test_tensorstore_image():
     """Test an image coming from a tensorstore array."""
+    ts = pytest.importorskip('tensorstore')
+
     data = ts.array(
         np.full(shape=(1024, 1024), fill_value=255, dtype=np.uint8)
     )
     layer = Image(data)
     assert np.all(layer.data == data)
+
+
+@pytest.mark.parametrize(
+    "start_position, end_position, view_direction, vector, expected_value",
+    [
+        # drag vector parallel to view direction
+        # projected onto perpendicular vector
+        ([0, 0, 0], [0, 0, 1], [0, 0, 1], [1, 0, 0], 0),
+        # same as above, projection onto multiple perpendicular vectors
+        # should produce multiple results
+        ([0, 0, 0], [0, 0, 1], [0, 0, 1], [[1, 0, 0], [0, 1, 0]], [0, 0]),
+        # drag vector perpendicular to view direction
+        # projected onto itself
+        ([0, 0, 0], [0, 1, 0], [0, 0, 1], [0, 1, 0], 1),
+        # drag vector perpendicular to view direction
+        # projected onto itself
+        ([0, 0, 0], [0, 1, 0], [0, 0, 1], [0, 1, 0], 1),
+    ],
+)
+def test_projected_distance_from_mouse_drag(
+    start_position, end_position, view_direction, vector, expected_value
+):
+    image = Image(np.ones((32, 32, 32)))
+    image._slice_dims(point=[0, 0, 0], ndisplay=3)
+    result = image.projected_distance_from_mouse_drag(
+        start_position,
+        end_position,
+        view_direction,
+        vector,
+        dims_displayed=[0, 1, 2],
+    )
+    assert np.allclose(result, expected_value)
+
+
+def test_rendering_init():
+    np.random.seed(0)
+    data = np.random.rand(10, 10, 10)
+    layer = Image(data, rendering='iso')
+
+    assert layer.rendering == ImageRendering.ISO.value
