@@ -7,6 +7,7 @@ from napari.layers.utils.stack_utils import (
     split_channels,
     stack_to_images,
 )
+from napari.utils.transforms import Affine
 
 
 def test_stack_to_images_basic():
@@ -16,7 +17,7 @@ def test_stack_to_images_basic():
     images = stack_to_images(stack, 1, colormap=None)
 
     assert isinstance(images, list)
-    assert images[0].colormap[0] == 'magenta'
+    assert images[0].colormap.name == 'magenta'
     assert len(images) == 2
 
     for i in images:
@@ -66,7 +67,7 @@ def test_stack_to_images_4_channels():
 
     assert isinstance(images, list)
     assert len(images) == 4
-    assert images[-2].colormap[0] == 'red'
+    assert images[-2].colormap.name == 'red'
     for i in images:
         assert type(stack) == type(i)
         assert i.data.shape == (128, 128)
@@ -112,7 +113,7 @@ def test_images_to_stack_with_scale():
 
     assert isinstance(stack, Image)
     assert stack.data.shape == (10, 3, 128, 128)
-    assert stack.colormap[0] == 'green'
+    assert stack.colormap.name == 'green'
     assert list(stack.scale) == [3, 1, 1, 1]
     assert list(stack.translate) == [1, 0, 2, 3]
 
@@ -132,7 +133,7 @@ def test_images_to_stack_none_scale():
 
     assert isinstance(stack, Image)
     assert stack.data.shape == (10, 3, 128, 128)
-    assert stack.colormap[0] == 'green'
+    assert stack.colormap.name == 'green'
     assert list(stack.scale) == [4, 1, 1, 1]
     assert list(stack.translate) == [0, 0, -1, 2]
 
@@ -156,6 +157,8 @@ def test_images_to_stack_none_scale():
             'blending': None,
             'visible': True,
             'multiscale': None,
+            'rotate': None,
+            'affine': None,
         },
         {
             'rgb': None,
@@ -224,3 +227,45 @@ def test_split_channels_missing_keywords():
     for d, meta, _ in result_list:
         assert d.shape == (128, 128)
         assert meta['blending'] == 'additive'
+
+
+def test_split_channels_affine_nparray(kwargs):
+    kwargs['affine'] = np.eye(3)
+    data = np.random.randint(0, 200, (3, 128, 128))
+    result_list = split_channels(data, 0, **kwargs)
+
+    assert len(result_list) == 3
+    for d, meta, _ in result_list:
+        assert d.shape == (128, 128)
+        assert np.array_equal(meta['affine'], np.eye(3))
+
+
+def test_split_channels_affine_napari(kwargs):
+    kwargs['affine'] = Affine(affine_matrix=np.eye(3))
+    data = np.random.randint(0, 200, (3, 128, 128))
+    result_list = split_channels(data, 0, **kwargs)
+
+    assert len(result_list) == 3
+    for d, meta, _ in result_list:
+        assert d.shape == (128, 128)
+        assert np.array_equal(meta['affine'].affine_matrix, np.eye(3))
+
+
+def test_split_channels_multi_affine_napari(kwargs):
+    kwargs['affine'] = [
+        Affine(scale=[1, 1]),
+        Affine(scale=[2, 2]),
+        Affine(scale=[3, 3]),
+    ]
+
+    data = np.random.randint(0, 200, (3, 128, 128))
+    result_list = split_channels(data, 0, **kwargs)
+
+    assert len(result_list) == 3
+    for idx, result_data in enumerate(result_list):
+        d, meta, _ = result_data
+        assert d.shape == (128, 128)
+        assert np.array_equal(
+            meta['affine'].affine_matrix,
+            Affine(scale=[idx + 1, idx + 1]).affine_matrix,
+        )
