@@ -113,7 +113,10 @@ def split_channels(
         ):
             kwargs[key] = iter(
                 ensure_sequence_of_iterables(
-                    val, n_channels, repeat_empty=True
+                    val,
+                    n_channels,
+                    repeat_empty=True,
+                    allow_none=True,
                 )
             )
         else:
@@ -238,7 +241,10 @@ def stack_to_images(stack: Image, axis: int, **kwargs) -> List[Image]:
 def split_rgb(stack: Image, with_alpha=False) -> List[Image]:
     """Variant of stack_to_images that splits an RGB with predefined cmap."""
     if not stack.rgb:
-        raise ValueError('Image must be RGB to use split_rgb')
+        raise ValueError(
+            trans._('Image must be RGB to use split_rgb', deferred=True)
+        )
+
     images = stack_to_images(stack, -1, colormap=('red', 'green', 'blue'))
     return images if with_alpha else images[:3]
 
@@ -281,5 +287,7 @@ def images_to_stack(images: List[Image], axis: int = 0, **kwargs) -> Image:
 def merge_rgb(images: List[Image]) -> List[Image]:
     """Variant of images_to_stack that makes an RGB from 3 images."""
     if not (len(images) == 3 and all(isinstance(x, Image) for x in images)):
-        raise ValueError("merge_rgb requires 3 images layers")
+        raise ValueError(
+            trans._("merge_rgb requires 3 images layers", deferred=True)
+        )
     return images_to_stack(images, axis=-1, rgb=True)
