@@ -1,20 +1,14 @@
-import os
 import csv
+import os
 from pathlib import Path
-import numpy as np
-from dask import array as da
 from tempfile import TemporaryDirectory
-from napari.utils import io
+
+import numpy as np
 import pytest
+import zarr
+from dask import array as da
 
-
-try:
-    import zarr
-
-    zarr_available = True
-except ImportError:
-    zarr_available = False
-
+from napari.utils import io
 
 # the following fixtures are defined in napari/conftest.py
 # single_png, two_pngs, irregular_images, single_tiff
@@ -118,7 +112,6 @@ def test_guess_zarr_path():
     assert not io.guess_zarr_path('no_zarr_suffix/data.png')
 
 
-@pytest.mark.skipif(not zarr_available, reason='zarr not installed')
 def test_zarr():
     image = np.random.random((10, 20, 20))
     with TemporaryDirectory(suffix='.zarr') as fout:
@@ -130,7 +123,6 @@ def test_zarr():
         np.testing.assert_array_equal(image, image_in)
 
 
-@pytest.mark.skipif(not zarr_available, reason='zarr not installed')
 def test_zarr_nested(tmp_path):
     image = np.random.random((10, 20, 20))
     image_name = 'my_image'
@@ -142,7 +134,6 @@ def test_zarr_nested(tmp_path):
     np.testing.assert_array_equal(image, image_in)
 
 
-@pytest.mark.skipif(not zarr_available, reason='zarr not installed')
 def test_zarr_multiscale():
     multiscale = [
         np.random.random((20, 20)),
